@@ -1,4 +1,4 @@
-# $Id: Directory.pm 40891 2014-01-27 10:42:11Z anton $
+# $Id: Directory.pm 41227 2014-03-26 13:47:17Z anton $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/pwdmodifier/libpwdmodifier/lib/UvT/PwdModifier/Directory.pm $
 
 use strict;
@@ -138,16 +138,18 @@ use Spiffy -Base;
 use Net::LDAP;
 use Net::LDAP::Filter;
 
-field 'cf', -init => 'die "no cf configured\n"';
+field 'cf';
 field 'ldap';
 field 'server';
 field 'retries';
 field 'capath';
 field 'bindAccountFilter';
+field 'bindAccount';
 field 'bindPassword';
+field 'ldapbase';
 field debug => 0;
 field usePagedResults => 0;
-
+field 'ssl' => 1;
 
 sub ldapError {
 	#callback heeft geen $self
@@ -159,13 +161,21 @@ sub ldapError {
 
 sub ldap_connection {
 	my $cf = $self->cf;
-	my $cfhash = $cf->hash();
+	my $cfhash = {};
+	$cfhash = $cf->hash() if $cf;
 
 	my $ldap = $self->ldap;
 	unless($ldap) {
 		my $server = $cfhash->{'ldaphost'} || $self->server;
-		my $capath = $cfhash->{'capath'} || $self->capath;
+		my $capath = $cfhash->{'capath'} || $self->capath || '/etc/ssl/certs/';
 		my $retries = $cfhash->{'ldapRetries'} || $self->retries || 1;
+
+		# N.B. Als gevolg van deze contructie moet ldap_connection altijd VOOR $self->ldapbase uit te lezen
+		my $ldapbase = $cfhash->{'ldapbase'} || $self->{ldapbase};
+		$self->ldapbase($ldapbase);
+		warn "ldapBASE: " . $self->ldapbase;
+
+		my $ssl = $self->ssl || 'on';
 		my $state;
 		my $debug = $self->debug;
 		warn "debug: $debug" if $debug;
@@ -180,30 +190,38 @@ sub ldap_connection {
 							  capath => $capath);
 		$state = $@;
 		die "Error connecting to $server: $state" unless $ldap;
-
-		$ldap->start_tls(verify => 'require', capath => $capath)
-			unless $ldap->cipher;
+		
+		unless ($ssl eq 'off') {
+			$ldap->start_tls(verify => 'require', capath => $capath)
+				unless $ldap->cipher;
+		}
 
 		$self->ldap($ldap);
 	}
-#	warn "About to bind" if $self->debug;
 	my $bindAccountFilter = $self->bindAccountFilter;
-	if ($bindAccountFilter) {
-		my $base = $cf->value('ldapbase');
-		my $msg = $ldap->search(base => $base, filter => $self->bindAccountFilter);
-		my @entries = $msg->entries;
+	my $bindAccount = $self->bindAccount;
 
-		return undef, "no match found for filter: '$bindAccountFilter'"
-			unless @entries;
+	if ($bindAccountFilter || $bindAccount) {
+		unless ($bindAccount) {
+			warn "Using filter: \"$bindAccountFilter\" to find bindAccount" if $self->debug;
+			my $base = $self->ldapbase;
+			my $msg = $ldap->search(base => $base, filter => $self->bindAccountFilter);
+			my @entries = $msg->entries;
+			
+			return undef, "no match found for filter: '$bindAccountFilter'"
+				unless @entries;
+			
+			return undef, "multiple entries found for filter: '$bindAccountFilter'"
+				if @entries > 1;
+			
+			$bindAccount = $entries[0]->dn();
+		}
 
-		return undef, "multiple entries found for filter: '$bindAccountFilter'"
-			if @entries > 1;
+		warn "bindAccount: " . $bindAccount if $self->debug; 
+		$ldap->bind($bindAccount, password => $self->bindPassword);
+		warn "Bind successful for: $bindAccount" if $self->debug; 
 
-		my $dn = $entries[0]->dn();
-		warn "Bind dn: $dn" if $self->debug; 
 
-		$ldap->bind($dn, password => $self->bindPassword);
-		warn "Bind successful for: $dn" if $self->debug; 
 	} else {
 		$ldap->bind;
 		warn "Anonymous bind successful" if $self->debug;
@@ -214,11 +232,10 @@ sub ldap_connection {
 sub ldap_search_filter {
 	my ($filter) = @_;
 	my $ldap = $self->ldap_connection();
-	warn "filter: $filter" if $self->debug;
-
 	my $cf = $self->cf;
-	my $base = $cf->value('ldapbase');
+	my $base = $self->ldapbase;
 	die $cf->error if $cf->error;
+	warn "filter: $filter, base: $base" if $self->debug;
 
 	my $search = $ldap->search(base => $base, filter => $filter);
 	my @entries = $search->entries;
@@ -293,7 +310,6 @@ sub getAccount {
 
 sub check_uvt_auth {
 	my ($admin, $auths) = @_;
-
 	my $ldap = $self->ldap_connection();
 
 	# (&(uid=bob)(|(uvt-auth=pwd_change/foo)(uvt-auth=pwd_change/foo/bar)))
@@ -312,8 +328,8 @@ sub getLdapEntriesByFilter {
 	$attrs = [] unless $attrs;
 	my $cf = $self->cf;
 
-	my $base = $cf->value('ldapbase');
 	my $ldap = $self->ldap_connection();
+	my $base = $self->ldapbase;
 
 	warn "getLdapEntriesByFilter; filter: $filter" if $self->debug;
 	my $entries =  $self->searchLdap(base => $base, filter => $filter, attrs => $attrs);
@@ -372,7 +388,7 @@ sub getEntriesByUID {
 	warn "getEntriesByUid: $uid" if $self->debug;
 	my $ldap = $self->ldap_connection();
 	my $cf = $self->cf;
-	my $base = $cf->value('ldapbase');
+	my $base = $self->ldapbase;
 	die $cf->error if $cf->error;
 
 	my $filter = bless(
@@ -403,13 +419,13 @@ sub getRcryptPassword {
 }
 
 sub toggleAccountState {
-	my ($attrib, $uid, $on, $label) = @_;
+	my ($attrib, $uid, $on, $label, $by) = @_;
 	my ($entry, $ldap) = $self->getEntryByUID($uid);
 
 	if ($on) {
 		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
 		$year += 1900; $mday +=1 ;
-		my $content = sprintf("$label at: $year-%02d-%02d %02d:%02d", $mon, $mday, $hour,$min);
+		my $content = sprintf("$label at: $year-%02d-%02d %02d:%02d $by", $mon, $mday, $hour,$min);
 		$entry->replace($attrib, $content);
 	} else {
 		$entry->delete($attrib);
